@@ -8,9 +8,9 @@
  * Functions for importing/exporting and converting SEXPs to mpz_t
  * 
  */
-void createMPZArray (SEXP input, mpz_t *myVec, unsigned long int sizevec, 
+void createMPZArray (SEXP input, mpz_t *myVec, std::size_t sizevec, 
                      std::string nameOfObject, bool negPoss = false);
 
-int myRaw (char* raw, mpz_t value, unsigned long int totals);
+int myRaw (char* raw, mpz_t value, std::size_t totals);
 
 #endif
