@@ -43,7 +43,7 @@ namespace PrimeCounting {
         
         int64_t sqrPrime = 9;
         int64_t lowerBnd = 0, upperBnd = segSize, myNum = 1;
-        unsigned long int p = 1;
+        std::size_t p = 1;
         
         for (; lowerBnd < flrMaxNum; lowerBnd += segSize) {
             upperBnd = lowerBnd + segSize;
@@ -219,7 +219,7 @@ namespace PrimeCounting {
             //                 log(x / 1e10) / (t * log(1.5))
             int64_t myRange = (piSqrtx - strt) + 1;
             int64_t lower = strt;
-            int64_t upper, chunk;
+            int64_t upper;
             RcppThread::ThreadPool pool(nThreads);
             std::vector<std::future<int64_t>> myFutures;
             
@@ -286,7 +286,7 @@ namespace PrimeCounting {
                 pool.join();
                 
             } else {
-                chunk = myRange / nThreads;
+                int64_t chunk = myRange / nThreads;
                 upper = lower + chunk - 1;
                 
                 for (int j = 0; j < (nThreads - 1); lower = upper, upper += chunk, ++j)
@@ -523,9 +523,9 @@ SEXP EratosthenesRcpp(SEXP Rb1, SEXP Rb2, SEXP RNumThreads, int maxCores, int ma
         CleanConvert::convertPrimitive(RNumThreads, nThreads, "nThreads");
     
     std::size_t numPrimes = 0u;
-    std::vector<unsigned long int> runningCount;
+    std::vector<std::size_t> runningCount;
     runningCount.push_back(0u);
-    unsigned long int numSects = nThreads;
+    std::size_t numSects = nThreads;
     bool Parallel = false;
     
     if (myMax > std::numeric_limits<int>::max()) {
