@@ -31,6 +31,6 @@ test_that("isPrimeRcpp generates correct numbers", {
 test_that("isPrimeRcpp produces appropriate error messages", {
     expect_error(isPrimeRcpp(0), "Each element in v must be a positive number")
     expect_error(isPrimeRcpp(-1), "Each element in v must be a positive number")
-    expect_error(isPrimeRcpp(2^53), "Each element in v must be less than")
+    expect_error(isPrimeRcpp(2^53), "The abs value of each element in v must be less than")
     expect_error(isPrimeRcpp("100000"), "must be of type numeric or integer")
 })
