@@ -33,9 +33,9 @@ test_that("primeFactorize generates correct numbers", {
 })
 
 test_that("primeFactorize produces appropriate error messages", {
-    expect_error(primeFactorize(2^53), "each element must be less than")
-    expect_error(primeFactorize(-2^53), "each element must be less than")
-    expect_error(primeFactorize(c(-2^53, 1:100)), "the abs value of each element must be less than")
+    expect_error(primeFactorize(2^53), "The abs value of each element in v must be less than")
+    expect_error(primeFactorize(-2^53), "The abs value of each element in v must be less than")
+    expect_error(primeFactorize(c(-2^53, 1:100)), "The abs value of each element in v must be less than")
     expect_error(primeFactorize("10"), "must be of type numeric or integer")
     expect_error(primeFactorize(100, namedList = "TRUE"), "Only logical values are supported for namedList")
 })
\ No newline at end of file
