@@ -53,7 +53,7 @@ test_that("primeSieve generates correct numbers", {
 
 test_that("primeSieve produces appropriate error messages", {
     expect_error(primeSieve(-1), "must be a positive")
-    expect_error(primeSieve(1,-1), "must be a positive number")
+    expect_error(primeSieve(1,-1), "must be a positive whole number")
     expect_error(primeSieve(1,2^53), "must be less than")
     expect_error(primeSieve(2^53), "must be less than")
     expect_error(primeSieve(2^53, 1), "must be less than")
